@@ -75,13 +75,7 @@ cInStream::cInStream(int Pid, spdif *dev, ctrl_t &up, cBounce *bPtr)
 
 cInStream::~cInStream(void)
 {
-    if (test_flag(ACTIVE))
-	Clear();
-    else {
-	bounce->flush();
-	ResetScan();
-	clear_flag(FAILED);
-    }
+    Clear();
 }
 
 void cInStream::Activate(bool onoff)
@@ -110,8 +104,10 @@ void cInStream::Activate(bool onoff)
 
 	clear_flag(ACTIVE);			// Set only above
 	do {
-	    bounce->flush();
-	    bounce->signal();
+	    if (bounce) {
+		bounce->flush();
+		bounce->signal();
+	    }
 	    wait.msec(10);			// More than 2ms
 	    if (!Active())
 		break;
@@ -128,7 +124,8 @@ void cInStream::Activate(bool onoff)
 		spdifDev->Close();
 		clear_flag(BOUNDARY);
 		curr->Reset();
-		bounce->leaveio();
+		if (bounce)
+		    bounce->leaveio();
 	    }
 	    clear_flag(RUNNING);		// Should not happen
 	}
@@ -225,8 +222,11 @@ out:
     ctrl.Lock();
     stream = NULL;
     ctrl.Unlock();
-    bounce->flush();
-    bounce->leaveio();
+    if (bounce) {
+	bounce->flush();
+	bounce->leaveio();
+    }
+    clear_flag(ACTIVE);
     clear_flag(RUNNING);
 }
 
@@ -237,8 +237,11 @@ void cInStream::Receive(uchar *b, int cnt)
     uint_8 start = 0;
     static uint_16 skip = 0;
 
+    if (test_setup(CLEAR))
+	goto out;
+
     if (!test_setup(ACTIVE)) {
-	if (test_flag(ACTIVE))
+	if (test_flag(RUNNING))
 	    Activate(false);
 	goto out;
     }
@@ -723,8 +726,10 @@ reset:
 
 void cInStream::Clear(void)
 {
-    Activate(false);
-    bounce->flush();
+    if (test_flag(RUNNING))
+	Activate(false);
+    if (bounce)
+	bounce->flush();
     ResetScan();
     clear_flag(FAILED);
 }
@@ -745,8 +750,6 @@ cChannelOutSPDif::cChannelOutSPDif(spdif &dev, ctrl_t &up, cBounce * bPtr, const
 {
     flags = 0;
     in = NULL;
-    replaying = false;
-    recording = false;
     bounce = bPtr;
 #if defined(VDRVERSNUM) && VDRVERSNUM <= 10317
     Player = NULL;
@@ -755,17 +758,23 @@ cChannelOutSPDif::cChannelOutSPDif(spdif &dev, ctrl_t &up, cBounce * bPtr, const
 
 cChannelOutSPDif::~cChannelOutSPDif(void)
 {
-#if defined(VDRVERSNUM) && VDRVERSNUM >= 10320
-    if (test_flag(ACTIVE))
-#endif
     Clear();
-    if (in) AttachReceiver(false);
 }
 
 bool cChannelOutSPDif::InTransferMode(void)
 {
     cDevice *PrimaryDevice = cDevice::PrimaryDevice();
 
+    // No primary device is always in transfer mode
+    if (!PrimaryDevice)
+	goto out;
+
+#if defined(VDRVERSNUM) && VDRVERSNUM >= 10342
+    // Just missed a few years
+    if (PrimaryDevice->Transferring())
+	goto out;
+#endif
+
     // Sanity check not be attached in transfer mode.
     // In this case the replay part should take the data
     if (PrimaryDevice != cDevice::ActualDevice())
@@ -773,27 +782,30 @@ bool cChannelOutSPDif::InTransferMode(void)
 
     // Next sanity check not be attached in transfer mode
     // from the primary device to the primary device.
-    // As metioned above the replay part should take the data
+    // As mentioned above the replay part should take the data
     if (PrimaryDevice == cTransferControl::ReceiverDevice())
 	goto out;
 
     return false;
 out:
-    sw.Lock();
-    if (in) AttachReceiver(false);
-    Channel = NULL;
-    sw.Unlock();
+    AudioOff();
     return true;
 }
 
 #if defined(VDRVERSNUM) && VDRVERSNUM >= 10321
+//
+// Stop current attached input filer thread if any and wake up the
+// switching thread if a new audio track was choosen.  The Apid
+// will be determined by GetCurrentAudioTrack() in this thread.
+//
 void cChannelOutSPDif::SetAudioTrack(int Index, const char * const *Tracks)
 {
+    cDevice *PrimaryDevice = cDevice::PrimaryDevice();
     const char* type = audioTypes[IEC_NONE];
     uint16_t apid = 0x1FFF;
     int num = 0;
 
-    if (replaying || recording)
+    if (!PrimaryDevice)
 	goto out;
 
     if (!Channel)
@@ -806,7 +818,7 @@ void cChannelOutSPDif::SetAudioTrack(int Index, const char * const *Tracks)
 	goto out;
 
     for (int i = ttAudioFirst; i <= ttDolbyLast; i++) {
-	const tTrackId *track = cDevice::PrimaryDevice()->GetTrack(eTrackType(i));
+	const tTrackId *track = PrimaryDevice->GetTrack(eTrackType(i));
 	if (!track || !track->id)
 	    continue;
 	if (Index == num) {
@@ -824,10 +836,11 @@ void cChannelOutSPDif::SetAudioTrack(int Index, const char * const *Tracks)
 	goto out;
 
     sw.Lock();
-    if (in) AttachReceiver(false);
-    if (!test_flag(RUNNING)) {
-	clear_flag(ACTIVE);
-	Activate(true);
+    if (!Replaying())
+    {
+	if (in) AttachReceiver(false);		// Do _not_ reset Channel
+	if (!test_flag(RUNNING))
+	    Activate(true);
     }
     sw.Unlock();
     ctrl.Lock();
@@ -836,7 +849,7 @@ void cChannelOutSPDif::SetAudioTrack(int Index, const char * const *Tracks)
 out:
     return;
 }
-#endif // VDRVERSNUM < 10321
+#endif // VDRVERSNUM >= 10321
 
 void cChannelOutSPDif::Activate(bool onoff)
 {
@@ -893,10 +906,7 @@ void cChannelOutSPDif::Action(void)
 	if (test_setup(CLEAR) || !test_setup(ACTIVE))
 	    break;
 
-	if (replaying || recording)
-	    continue;
-
-	if (!Channel)
+	if (Replaying())
 	    continue;
 
 	if (!GetCurrentAudioTrack(apid, type))		// Get audio pid and type
@@ -916,28 +926,17 @@ void cChannelOutSPDif::Action(void)
 #if defined(VDRVERSNUM) && VDRVERSNUM < 10300
     dsyslog("CHANNELOUT: Switching bitstream thread ended (pid=%d)", getpid());
 #endif
+    clear_flag(ACTIVE);
     clear_flag(RUNNING);
 }
 
-void cChannelOutSPDif::AudioSwitch(uint_16 apid, const char* type)
+void cChannelOutSPDif::AudioSwitch(uint_16 apid, const char* type, const cChannel * channel)
 {
     sw.Lock();
-
-    if (!test_setup(ACTIVE)) {
-	Activate(false);
-	if (in) AttachReceiver(false);
-	Channel = NULL;			// Reset Channel
-	goto out;
-    } else {
-	if (!test_flag(RUNNING)) {
-	    clear_flag(ACTIVE);
-	    Activate(true);
-	}
-    }
-
     if (in) AttachReceiver(false);	// Do _not_ reset Channel
 
-    if (replaying || recording) {
+    if (Replaying())
+    {
 #if defined(VDRVERSNUM) && VDRVERSNUM > 10317
 	goto out;	
 #else
@@ -950,6 +949,12 @@ void cChannelOutSPDif::AudioSwitch(uint_16 apid, const char* type)
 	}
 #endif
     } else {
+	if (channel)
+	    Channel = channel;
+
+	if (!Channel)
+	    goto out;
+
 	Apid = apid;
 	audioType = type;
 
@@ -958,8 +963,6 @@ void cChannelOutSPDif::AudioSwitch(uint_16 apid, const char* type)
 
 	if (Apid && Apid < 0x1FFF)
 	    AttachReceiver(true);
-	else
-	    test_setup(RESET);
     }
 out:
     sw.Unlock();
@@ -967,6 +970,16 @@ out:
     return;
 }
 
+void cChannelOutSPDif::AudioOff(void)
+{
+    sw.Lock();
+    Channel = NULL;		// Reset Channel
+    Apid = 0x1FFF;		// Reset Apid
+    audioType = audioTypes[IEC_NONE];	// ... and its type
+    if (in) AttachReceiver(false);
+    sw.Unlock();
+}
+
 void cChannelOutSPDif::AttachReceiver(bool onoff)
 {
     cDevice *PrimaryDevice = cDevice::PrimaryDevice();
@@ -976,9 +989,14 @@ void cChannelOutSPDif::AttachReceiver(bool onoff)
 
     if (in) {
 	in->Clear();
+#if defined(VDRVERSNUM) && VDRVERSNUM > 10325
+	if (PrimaryDevice)
+	    PrimaryDevice->Detach(in);
+#endif
 	delete in;
 	in = NULL;
-	bounce->flush();
+	if (bounce)
+	    bounce->flush();
 	Apid = 0x1FFF;
 	audioType = audioTypes[IEC_NONE];
     }
@@ -989,6 +1007,9 @@ void cChannelOutSPDif::AttachReceiver(bool onoff)
     if (!PrimaryDevice)
 	goto out;
 
+    if (PrimaryDevice->Replaying())
+	goto out;
+
     in = new cInStream(Apid, spdifDev, setup, bounce);
     if (!in) {
 	esyslog("ERROR: out of memory");
@@ -1004,12 +1025,15 @@ out:
 
 void cChannelOutSPDif::ChannelSwitch(const cDevice *Device, int channelNumber)
 {
-    cDevice *PrimaryDevice = cDevice::PrimaryDevice();
     uint_16 apid;
     const char * type;
+    cChannel * channel;
 
-    if (!PrimaryDevice)
+    if (!test_setup(ACTIVE)) {
+	Clear();
 	goto out;
+    } else if (!test_flag(RUNNING))
+	Activate(true);
 
     // LiveView is only on the primary device possible, isn't it?
     if (!Device->IsPrimaryDevice())
@@ -1020,75 +1044,90 @@ void cChannelOutSPDif::ChannelSwitch(const cDevice *Device, int channelNumber)
 
     // If we're noticed only for channel switch we're done.
     // Also during a replay we're done.
-    if (!channelNumber || replaying) {
+    if (!channelNumber || Replaying()) {
 	//
 	// Problem: Channel switches for recording on the same
 	// transponder will cause a small audio pause.
 	//
-	sw.Lock();
-	if (in) AttachReceiver(false);
-	Channel = NULL;			// Reset Channel
-	sw.Unlock();
+	AudioOff();
 	goto out;
     }
 
     // Now get the current live channel but _not_ the given
     // channel number (maybe this one is for recording).
-    Channel = Channels.GetByNumber(cDevice::CurrentChannel());
+    channel = Channels.GetByNumber(cDevice::CurrentChannel());
 
     // Oops, no channel is no audio
-    if (!Channel)
+    if (!channel) {
+	AudioOff();
+	goto out;
+    }
+
+    // New channel, new audio
+    if (Channel != channel)
+	AudioOff();
+
+    if (!GetCurrentAudioTrack(apid, type, channel))
 	goto out;
 
-    Apid = 0x1FFF;			// Reset Apid
-    audioType = audioTypes[IEC_NONE];	// ... and its type
-    if (!GetCurrentAudioTrack(apid, type, Channel))
-	goto out;
-
-    AudioSwitch(apid, type);		// Set Apid and its type
+    AudioSwitch(apid, type, channel);		// Set Channel, Apid and its type
 out:
     return;
 }
 
-void cChannelOutSPDif::Recording(const cDevice  *Device,  const char *Name)
+#if VDRVERSNUM >= 10338
+void cChannelOutSPDif::Recording(const cDevice *Device, const char *Name, const char *FileName, bool On)
+#else
+void cChannelOutSPDif::Recording(const cDevice *Device, const char *Name)
+#endif
 {
-    cDevice *PrimaryDevice = cDevice::PrimaryDevice();
+    sw.Lock();
+    cDevice *PrimaryDevice  = cDevice::PrimaryDevice();
+    cDevice *ReceiverDevice = cTransferControl::ReceiverDevice();
+#if VDRVERSNUM < 10338
+    bool On = (Name != NULL);
+#endif
 
     if (!PrimaryDevice)
 	goto out;
 
-    if (Device != PrimaryDevice)
-	goto out;
+    if (PrimaryDevice->Replaying())
+	goto out;	// Nothing attached
 
-    recording = (Name != NULL);
+    if (!ReceiverDevice)
+	goto out;	// No receiver device
 
-    if (!recording)
-	goto out;
+    if (PrimaryDevice != ReceiverDevice)
+	goto out;	// Two or more cards
 
-    sw.Lock();
-    if (in) AttachReceiver(false);
-    Channel = NULL;			// Reset Channel
-    sw.Unlock();
+    if (On) {
+	Channel = NULL;			// Reset Channel
+	if (in) AttachReceiver(false);
+    }
 out:
+    sw.Unlock();
     IfNeededMuteSPDIF();		// On close the S/P-DIF is not muted anymore
 }
 
+#if VDRVERSNUM >= 10338
+void cChannelOutSPDif::Replaying(const cControl *Control, const char *Name, const char *FileName, bool On)
+#else
 void cChannelOutSPDif::Replaying(const cControl *Control, const char *Name)
+#endif
 {
-    replaying = (Name != NULL);
-
-#if defined(VDRVERSNUM) && VDRVERSNUM <= 10317
-    Player = (static_cast<const cControlPerm *>(Control))->Player();
+    sw.Lock();
+#if VDRVERSNUM < 10338
+    bool On = (Name != NULL);
 #endif
 
-    if (!replaying)
-	goto out;
-
-    sw.Lock();
-    if (in) AttachReceiver(false);
-    Channel = NULL;			// Reset Channel
+    if (On) {
+#if defined(VDRVERSNUM) && VDRVERSNUM <= 10317
+	Player = (static_cast<const cControlPerm *>(Control))->Player();
+#endif
+	Channel = NULL;			// Reset Channel
+	if (in) AttachReceiver(false);
+    }
     sw.Unlock();
-out:
     IfNeededMuteSPDIF();		// On close the S/P-DIF is not muted anymore
 }
 
@@ -1116,21 +1155,21 @@ bool cChannelOutSPDif::GetCurrentAudioTrack(uint_16 &apid, const char* &type)
 {
     bool ret = false;
 
-    if (replaying || recording)
-	goto out;
-
     if (!Channel)
 	goto out;
 
     if (InTransferMode())
 	goto out;
 
+    if (Replaying())
+	goto out;
+
     ret = GetCurrentAudioTrack(apid, type, Channel);
 out:
     return ret;
 }
 
-bool cChannelOutSPDif::GetCurrentAudioTrack(uint_16 &apid, const char* &type, cChannel *channel)
+bool cChannelOutSPDif::GetCurrentAudioTrack(uint_16 &apid, const char* &type, const cChannel *channel)
 {
 #if defined(VDRVERSNUM) && VDRVERSNUM > 10317
     cDevicePerm *PrimaryDevice = static_cast<cDevicePerm *>(cDevice::PrimaryDevice());
@@ -1211,7 +1250,8 @@ out:
 
 bool cChannelOutSPDif::AudioTrack(int num, const char* type, int &apid)
 {
-    if (replaying || recording) {
+    if (Replaying())
+    {
 #if defined(VDRVERSNUM) && VDRVERSNUM > 10317
 	apid = 0x1FFF;
 #else
@@ -1253,7 +1293,8 @@ out:
 
 uint_16 cChannelOutSPDif::AudioPid(void)
 {
-    if (replaying || recording) {
+    if (Replaying())
+    {
 #if defined(VDRVERSNUM) && VDRVERSNUM > 10317
 	return 1;
 #else
@@ -1271,7 +1312,7 @@ uint_16 cChannelOutSPDif::AudioPid(void)
 
 const char* cChannelOutSPDif::AudioType(void)
 {
-    if (replaying || recording)
+    if (Replaying())
 	return audioTypes[IEC_NONE];
     if (in)
 	return in->AudioType();
@@ -1280,7 +1321,7 @@ const char* cChannelOutSPDif::AudioType(void)
 
 void cChannelOutSPDif::Clear(void)
 {
-    Activate(false);
-    if (in) AttachReceiver(false);
-    Channel = NULL;
+    if (test_flag(RUNNING))
+	Activate(false);
+    AudioOff();
 }
