@@ -142,19 +142,17 @@ private:
 #if defined(VDRVERSNUM) && VDRVERSNUM <= 10317
     cPlayer *Player;
 #endif
-    bool replaying;
-    bool recording;
     static const char *audioType;
     static uint_16 Apid;
     cMutex sw, ctrl;
     cCondVar watch;
-    cChannel *Channel;
+    const cChannel *Channel;			// Active Live Channel if any
     virtual void IfNeededMuteSPDIF(void);
     static cBounce * bounce;
     cPsleep wait;
     bool InTransferMode(void);
     bool GetCurrentAudioTrack(uint_16 &apid, const char* &type);
-    static bool GetCurrentAudioTrack(uint_16 &apid, const char* &type, cChannel *channel);
+    static bool GetCurrentAudioTrack(uint_16 &apid, const char* &type, const cChannel *channel);
     virtual void AttachReceiver(bool onoff);
 protected:
     const char *const SPDIFmute;
@@ -166,16 +164,25 @@ protected:
     virtual void Action(void);
     virtual void Activate(bool on);
     virtual void ChannelSwitch(const cDevice *Device, int ChannelNumber);
+#if VDRVERSNUM >= 10338
+    virtual void Recording(const cDevice *Device, const char *Name, const char *FileName, bool On);
+    virtual void Replaying(const cControl *Control, const char *Name, const char *FileName, bool On);
+#else
     virtual void Recording(const cDevice  *Device,  const char *Name);
     virtual void Replaying(const cControl *Control, const char *Name);
+#endif
 public:
     cChannelOutSPDif(spdif &dev, ctrl_t &up, cBounce * bPtr, const char *script);
     ~cChannelOutSPDif(void);
     bool    AudioTrack(int num, const char* type, int &apid);
     uint_16 AudioPid(void);
     const char * AudioType(void);
-    virtual void AudioSwitch(uint_16 apid, const char* type);
-    const bool Replaying(void) const { return replaying; };
+    virtual void AudioSwitch(uint_16 apid, const char* type, const cChannel* channel = NULL);
+    virtual void AudioOff(void);
+    const bool Replaying(void) const {
+	cDevice *PrimaryDevice = cDevice::PrimaryDevice();
+	return PrimaryDevice ? PrimaryDevice->Replaying() : true;
+    };
     virtual void Clear(void);
 };
 
