@@ -203,8 +203,6 @@ inline ssize_t cMP2::Sample(uint_8 *data, size_t cnt)
 
     if (len > pcm->length)
 	len = pcm->length;
-    else
-	dsyslog("MP2PCM: ** PCM frame length larger than burst size **");
 
 #if !defined(_GNU_SOURCE)
 # define pcm_cast(a)     (unsigned char)((a) & 0xff)
@@ -712,65 +710,75 @@ resync:
 		c.syncword = (c.syncword << 8) | *buf++;
 	    }
 
-	    i.ext   = ((c.syncword & 0x0010) >> 4) == 0;
-	    i.lsf   = ((c.syncword & 0x0008) >> 3) == 0;
-	    i.layer = 4 - ((c.syncword & 0x0006) >> 1);
+	    if (c.buffer_gard && (c.buffer_gard == MAD_BUFFER_GUARD))
+		c.buffer_gard -= 2;
+	    else
+		c.buffer_gard = 0;
+
+	    c.sync.ext   = ((c.syncword & 0x0010) >> 4) == 0;
+	    c.sync.lsf   = ((c.syncword & 0x0008) >> 3) == 0;
+	    c.sync.layer = 4 - ((c.syncword & 0x0006) >> 1);
 
 	    // Sanity check for mpeg audio
-	    if ((i.layer == 4) || (!i.lsf && i.ext)) {	// Combination
+	    if ((c.sync.layer == 4) || (!c.sync.lsf && c.sync.ext)) {	// Combination
 		c.bfound = 0;
 		c.syncword = 0x001f;
-		memset(&i, 0, sizeof(mp2info_t));
-		goto resync;				// ... not valid
+		memset(&c.sync, 0, sizeof(mp2info_t));
+		goto resync;					// ... not valid
 	    }
 
 	    c.bfound = 2;
 	case 2:
 	    if (buf >= tail)
 		goto out;
-	    {						// Frame size calculation
+	    {							// Frame size calculation
 		uint_32 bit_rate;
 		int index;
 		uint_8 padding;
-							// Sample rate index
-		if ((index = ((*buf) & 0x0c)) == 0x0c) {
+
+		if (c.buffer_gard && (c.buffer_gard == MAD_BUFFER_GUARD - 2))
+		     c.buffer_gard--;
+		else
+		     c.buffer_gard = 0;
+
+		if ((index = ((*buf) & 0x0c)) == 0x0c) {	// Sample rate index
 		    c.bfound = 0;
 		    c.syncword = 0x001f;
-		    memset(&i, 0, sizeof(mp2info_t));
-		    goto resync;			// ... not valid
+		    memset(&c.sync, 0, sizeof(mp2info_t));
+		    goto resync;				// ... not valid
 		}
 		index >>= 2;
 
-		i.sampling_rate = samplerate_table[index];
-		if (i.lsf) {				// Sample rate
-		    i.sampling_rate /= 2;
-		    if (i.ext)
-			i.sampling_rate /= 2;
+		c.sync.sampling_rate = samplerate_table[index];
+		if (c.sync.lsf) {				// Sample rate
+		    c.sync.sampling_rate /= 2;
+		    if (c.sync.ext)
+			c.sync.sampling_rate /= 2;
 		}
-							// Bit rate index
-		if ((index = ((*buf) & 0xf0)) == 0xf0) {
+
+		if ((index = ((*buf) & 0xf0)) == 0xf0) {	// Bit rate index
 		    c.bfound = 0;
 		    c.syncword = 0x001f;
-		    memset(&i, 0, sizeof(mp2info_t));
-		    goto resync;			// ... not valid
+		    memset(&c.sync, 0, sizeof(mp2info_t));
+		    goto resync;				// ... not valid
 		}
 		index >>= 4;
 
-		if (i.lsf)
-		    bit_rate = bitrate_table[3 + (i.layer >> 1)][index];
+		if (c.sync.lsf)
+		    bit_rate = bitrate_table[3 + (c.sync.layer >> 1)][index];
 		else
-		    bit_rate = bitrate_table[i.layer - 1][index];
+		    bit_rate = bitrate_table[c.sync.layer - 1][index];
 
 		padding = ((*buf) & 0x02) >> 1;
 
-		if (i.layer == 1) {
-		    c.fsize = 4*((12 * bit_rate / i.sampling_rate) + padding);
+		if (c.sync.layer == 1) {
+		    c.fsize = 4*((12 * bit_rate / c.sync.sampling_rate) + padding);
 		} else {
-		    const uint_16 slots = ((i.layer == 3) && i.lsf) ? 72 : 144;
-		    c.fsize = (slots * bit_rate / i.sampling_rate) + padding;
+		    const uint_16 slots = ((c.sync.layer == 3) && c.sync.lsf) ? 72 : 144;
+		    c.fsize = (slots * bit_rate / c.sync.sampling_rate) + padding;
 		}
 	    }
-	    debug_mp2("count_mp2_frame: %d %d\n", i.layer, c.fsize);
+	    debug_mp2("count_mp2_frame: %d %d\n", c.sync.layer, c.fsize);
 	    buf++;
 	    c.bfound++;
 	default:
@@ -779,21 +787,31 @@ resync:
     }
 
     while (c.bfound < c.fsize) {
-	register int skip, pay;
+	int skip, pay;
 	if ((skip = tail - buf) <= 0)
 	    goto out;
 	if (skip > (pay = c.fsize - c.bfound))
 	    skip = pay;
 	c.bfound += skip;
 	buf	 += skip;
+
+	if (c.buffer_gard) {				// Decrease also buffer guard
+	    if (skip > c.buffer_gard)
+		skip = c.buffer_gard;
+	    if ((c.buffer_gard -= skip) == 0)		// ... here and maybe we have
+		ret = true;				// crossed a frame and the guard
+	}
     }
 
-    if (c.fsize && (c.bfound == c.fsize)) {
+    if (c.fsize && (c.bfound >= c.fsize)) {
 	c.bfound = 0;
 	c.syncword = 0x001f;
-	memset(&i, 0, sizeof(mp2info_t));
-	ret = true;
+	memset(&c.sync, 0, sizeof(mp2info_t));
 	debug_mp2("count_mp2_frame: frame crossed\n");
+	if (test_flags(MP2SPDIF))
+	    ret = true;
+	else
+	    c.buffer_gard = MAD_BUFFER_GUARD;		// We have to await the buffer guard
 	if (buf >= tail)
 	    goto out;
 	goto resync;
