@@ -61,9 +61,6 @@ cReplayOutSPDif::cReplayOutSPDif(spdif &dev, ctrl_t &up, cBounce * bPtr, const c
 
 cReplayOutSPDif::~cReplayOutSPDif(void)
 {
-#if defined(VDRVERSNUM) && VDRVERSNUM >= 10320
-    if (test_flag(ACTIVE))
-#endif
     Clear();
 }
 
@@ -94,8 +91,10 @@ void cReplayOutSPDif::Activate(bool onoff)
 
 	clear_flag(ACTIVE);			// Set only above
 	do {
-	    bounce->flush();
-	    bounce->signal();
+	    if (bounce) {
+		bounce->flush();
+		bounce->signal();
+	    }
 	    wait.msec(10);			// More than 2ms
 	    if (!Active())
 		break;
@@ -112,7 +111,8 @@ void cReplayOutSPDif::Activate(bool onoff)
 		spdifDev->Close();
 		clear_flag(BOUNDARY);
 		curr->Reset();
-		bounce->leaveio();
+		if (bounce)
+		    bounce->leaveio();
             }
 	    clear_flag(RUNNING);		// Should not happen
 	}
@@ -128,6 +128,9 @@ out:
 
 void cReplayOutSPDif::Action(void)
 {
+    if (test_setup(CLEAR))
+	return;
+
     set_flag(RUNNING);
     ctr.Lock();
     iec60958 *curr = stream;
@@ -194,8 +197,11 @@ out:
     ctr.Lock();
     stream = NULL;
     ctr.Unlock();
-    bounce->flush();
-    bounce->leaveio();
+    if (bounce) {
+	bounce->flush();
+	bounce->leaveio();
+    }
+    clear_flag(ACTIVE);
     clear_flag(RUNNING);
 }
 
@@ -530,6 +536,9 @@ void cReplayOutSPDif::Play(const uchar *b, int cnt)
     iec60958* curr = stream;
     ctr.Unlock();
 
+    if (test_setup(CLEAR))
+	goto out;
+
     if (!test_setup(ACTIVE)) {
 	if (test_flag(ACTIVE))
 	    Clear();
@@ -692,11 +701,8 @@ void cReplayOutSPDif::Play(const uchar *b, int cnt)
 	if (curr) {
 	    const uint_8 * buf = (const uint_8 *)&b[off];
 	    const size_t transmit = min(len - off, cnt);
-#if 1
 	    const bool start = curr->Count(buf, buf+transmit);
-#else
-	    const bool start = true;
-#endif
+
 	    //
 	    // Initial synchronization
 	    //
@@ -776,19 +782,20 @@ void cReplayOutSPDif::Clear(void)
     cDevice *PrimaryDevice = cDevice::PrimaryDevice();
 
     debug("cReplayOutSPDif::Clear() called\n");
-    Activate(false);
+    if (test_flag(RUNNING))
+	Activate(false);
     ctr.Lock();
     iec60958* curr = stream;
+    stream = NULL;
     ctr.Unlock();
     if (curr) {
 	LOCK_THREAD;
 	curr->Reset();
-	bounce->flush();
-	bounce->signal();
+	if (bounce) {
+	    bounce->flush();
+	    bounce->signal();
+	}
     }
-    ctr.Lock();
-    stream = NULL;
-    ctr.Unlock();
     clear_flag(BOUNDARY);
     clear_setup(STILLPIC);
     if (PrimaryDevice)
