@@ -184,6 +184,9 @@ void spdif::Forward(const uint_8 *const data,
     frame_t pcm;
     off_t offset = 0;
 
+    if (test_setup(CLEAR))
+	goto xout;
+
     if (!out || !stream)
 	goto xout;
 
@@ -203,6 +206,14 @@ void spdif::Forward(const uint_8 *const data,
 		uint_32 duration = (B2F(pcm.size)*1000)/stream->SampleRate();
 		int mcnt;
 
+		//
+		// The Mpeg audio decoder requires always two data frames (one full
+		// and the start of the next frame), which we have to receive first,
+		// to be able to decode the first data frame. Therefore we loose the
+		// duration of the almost first resulting PCM frame.
+		//
+		if (opt.audio) duration = 1;
+
 		//
 		// If current STC value is not valid, we try next frame
 		// to get the correct pts offset we need the duration of
@@ -296,28 +307,21 @@ void spdif::Forward(const uint_8 *const data,
 		// This recovers the underrun in the case we're getting
 		// more input.
 		//
+
 		if (ctrlbits & FL_VRPERIOD) {
 
 		    Hold(thread);		// Hold the lock on the calling thread
 
 		    if (opt.audio) {
-#if 0
-			// We're playing real PCM data, therefore we play
-			// some single stereo samples twice.
 
-			if (pcm.size >= 4) {
-			    frame_t sample = { pcm.burst, 4, 4 };
-			    burst(sample);
-			}
-#else
 			if (!(repeat = (++repeat) % 4)) {
 			    // Use a wait frame for filling
 			    const frame_t fill = stream->Frame(PCM_WAIT2);
-			    // stream->SetErr();
+			    stream->SetErr();
 			    burst(fill);
-			    // stream->ClearErr();
+			    stream->ClearErr();
 			}
-#endif
+
 		    } else {
 			// Let us play with the error detection of the receiver and
 			// send some of the current bursts twice with error bit set.
@@ -331,16 +335,20 @@ void spdif::Forward(const uint_8 *const data,
 
 		    Unhold();			// Do not hold lock on calling thread
 
-		    // FL_VRPERIOD will be removed by external call of
+		    // FL_UNDERRUN will be removed by external call of
 		    // Synchronize() check()ing the buffers state
 
 		} else {
 		    // This also recovers the underrun, nevertheless the
 		    // stream to the receiver will get short leaks.
 
+		    Hold(thread);		// Hold the lock on the calling thread
+
 		    snd_pcm_drain(out);
 		    snd_pcm_prepare(out);
 		    ctrlbits &= ~FL_UNDERRUN;
+
+		    Unhold();			// Do not hold lock on calling thread
 		}
 
 	    } else if (ctrlbits & FL_OVERRUN) {
@@ -438,7 +446,7 @@ void spdif::burst(const frame_t &pcm)
 		if (stream)
 		    stream->Clear();
 		delay = 0;
-		break;
+		goto xout;
 	    case -ESTRPIPE:
 		xsuspend();
 		continue;
@@ -449,7 +457,7 @@ void spdif::burst(const frame_t &pcm)
 		continue;
 	    default:
 		esyslog("S/P-DIF: snd_pcm_writei returned error: %s", snd_strerror(res));
-		break;
+		goto xout;
 	    }
 	}
 
@@ -1070,6 +1078,9 @@ bool spdif::Synchronize(class cBounce *bounce)
     bool ready = true;
     int wait;
 
+    if (test_setup(CLEAR))
+	return false;
+
     if (ctrlbits & FL_FIRST)
 	goto do_wait;
 
