@@ -964,7 +964,7 @@ static off_t scan_c0(uint_8* const buf, const off_t rest, const uint_16 flags, c
 	} else {
 	    const uint_16 slots = ((layer == 3) && lsf) ? 72 : 144;
 	    size  = (slots * bitrate / rate) + padding;
-	    bytes = 4608;
+	    bytes = 32 * slots;
 	}
 
 	uint_64 lpts;
@@ -1129,7 +1129,7 @@ static off_t scan_bd(uint_8* const buf, const off_t rest, const uint_16 flags, c
 		    nl;
 		    N("%8llu %02X", deep+handle.Offset(), (ub & 0xf8));
 		    N(" substream(PCM), len=%d, flags:", len);
-		    handle += 3;
+		    handle += 4;
 		    for (int i = 0; i < 3; i++)
 			 N(" %02x", (uint_8)handle++);
 		} else if (len) {
@@ -1138,7 +1138,7 @@ static off_t scan_bd(uint_8* const buf, const off_t rest, const uint_16 flags, c
 		    nl;
 		    N("%8llu %02X", deep+handle.Offset(), (ub & 0xf8));
 		    N(" substream(PCM), rest=%d, flags", len);
-		    handle += 3;
+		    handle += 4;
 		    for (int i = 0; i < 3; i++)
 			 N(" %02x", (uint_8)handle++);
 		}
